{
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, 2000-2001 Mats Asplund
----------------------------------------------------------

Component Name: TmWebScrollTable
        Author: Mats Asplund
      Creation: 2003-03-20
       Version: 1.0
   Description: A component that generates HTML-tables with
                a scrollable bodypart.
        E-mail: masprod@telia.com
          Site: http://go.to/masdp
  Legal issues: All rights reserved 2003 by Mats Asplund

Usage:
  This software is provided 'as-is', without any express or
  implied warranty.  In no event will the author be held liable
  for any  damages arising from the use of this software.

  Permission is granted to anyone to use this software for any
  purpose, including commercial applications, and to alter it
  and redistribute it freely, subject to the following
  restrictions:

  1. The origin of this software must not be misrepresented,
     you must not claim that you wrote the original software.
     If you use this software in a product, an acknowledgment
     in the product documentation would be appreciated but is
     not required.

  2. Altered source versions must be plainly marked as such, and
     must not be misrepresented as being the original software.

  3. This notice may not be removed or altered from any source
     distribution.

  4. If you decide to use this software in any of your applications.
     Send me an EMail and tell me about it.

Quick Reference:
  TmWebScrollTable inherits from TComponent.
  TWebScrollTable inherits from TDSTableProducer.

  Key-Properties:
    WebScrollTable:  Here you can get access to the underlying propertys for
                     TWebScrollTable.
    DataSet:         Could be a query, stored procedure or table.

    Put this three files in the same folder:
    IFrameFile:      Filename for the resulting file with one IFrame for the
                     static tableheader, and one for the scrollable tablebody.
    HeaderFile:      Filename for the file containg the tableheader.
    BodyFile:        Filename for the file containg the tablebody.

    Width:           The table width. The header with will be scaled down by
                     ScrollBarOffset since there's no header scrollbar.
    HeaderHeight:    The Header height.
    BodyHeight:      The Body height;
    MarginWidth:     The horizontal table position in the IFrame.
    MarginHeight:    The vertical table position in the IFrame.
    ScrollBarOffset: Use this to get the table header one scrollbar smaller than
                     the body. Default value is 17, but may be browser dependent.

  Key-Events:
    OnFormatCell:    Use this event to further customize your table cells.
                     Note the Header parameter. When true the Header formatting
                     is taking place, if false the Body formatting is taking
                     place. Changes made here can override the basic formatting
                     done by the component.
                     Which is:
                       VAlign left.
                       For the header:
                       Every Row > 0 will be written with white font on white
                       background.
                       For the body:
                       Row 0 will be written with white font on white background.
  Key-Methods:
    Execute: When executing, a HTML-file with two IFrames will be created. One
             for the header and one for the body.
             The Dataset will be closed/opened and the formatted headercontent
             will be written to the HeaderFile.
             The Dataset will be closed/opened once again, and the formatted
             bodycontent will be written to the BodyFile.

--------------------------------------------------------------------------------
}
unit mWebScrollTable;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, db, dbweb, httpprod;

type
  TOnFormatCellEvent = procedure (Sender: TObject; Header: boolean; CellRow,
  CellColumn: integer; var BgColor: THTMLBgColor; var Align: THTMLAlign;
  var VAlign: THTMLVAlign; var CustomAttrs, CellData: string) of object;

  TWebScrollTable = class(TDSTableProducer)
  private
    FDataSet: TDataSet;
  protected
    function GetDataSet: TDataSet; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetDataSet(ADataSet: TDataSet); override;
    property DataSet;
  public
    function Content: string; override;
  published
    property Caption;
    property CaptionAlignment;
    property Columns;
    property Footer;
    property Header;
    property RowAttributes;
    property TableAttributes;
  end;

  TmWebScrollTable = class(TComponent)
  private
    fIFrames: string;
    fBodyFile: TFileName;
    fHeaderFile: TFileName;
    fIFrameFile: TFileName;
    fIFrame: TStringList;
    fWidth: integer;
    fMarginHeight: integer;
    fHeaderHeight: integer;
    fMarginWidth: integer;
    fBodyHeight: integer;
    fWebScrollTable: TWebScrollTable;
    fHeader: boolean;
    fAbout: string;
    fOnFormatCell: TOnFormatCellEvent;
    fScrollBarOffset: integer;
    procedure SetCop(Value: string);
    function GetDataSet: TDataSet;
    procedure SetDataSet(const Value: TDataSet);
    procedure SetBodyFile(const Value: TFileName);
    procedure SetHeaderFile(const Value: TFileName);
    procedure SetIFrameFile(const Value: TFileName);
    procedure FormatCell(Sender: TObject; CellRow, CellColumn: integer;
      var BgColor: THTMLBgColor; var Align: THTMLAlign;
      var VAlign: THTMLVAlign; var CustomAttrs, CellData: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
    property IFrames: string read fIFrames write fIFrames;
  published
    property WebScrollTable: TWebScrollTable read fWebScrollTable write fWebScrollTable;
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property IFrameFile: TFileName read fIFrameFile write SetIFrameFile;
    property HeaderFile: TFileName read fHeaderFile write SetHeaderFile;
    property BodyFile: TFileName read fBodyFile write SetBodyFile;
    property Width: integer read fWidth write fWidth;
    property HeaderHeight: integer read fHeaderHeight write fHeaderHeight;
    property BodyHeight: integer read fBodyHeight write fBodyHeight;
    property MarginWidth: integer read fMarginWidth write fMarginWidth;
    property MarginHeight: integer read fMarginHeight write fMarginHeight;
    property ScrollBarOffset: integer read fScrollBarOffset write fScrollBarOffset;
    property OnFormatCell: TOnFormatCellEvent read fOnFormatCell write fOnFormatCell;
    property About: string read fAbout write SetCop;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmWebScrollTable]);
end;

constructor TmWebScrollTable.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fWebScrollTable:= TWebScrollTable.Create(self);
  fIFrame:= TStringList.Create;
  fWebScrollTable.OnFormatCell:= FormatCell;
  fBodyFile:= 'body.htm';
  fBodyHeight:= 300;
  fHeaderFile:= 'header.htm';
  fHeaderHeight:= 25;
  fIFrameFile:= 'table.htm';
  fMarginHeight:= 0;
  fMarginWidth:= 0;
  fScrollBarOffset:= 17;
  fWidth:= 800;
  fAbout:= 'Version 1.0, 2003 Mats Asplund';
end;

destructor TmWebScrollTable.Destroy;
begin
  fWebScrollTable.Free;
  fIFrame.Free;
  inherited Destroy;
end;

procedure TmWebScrollTable.Execute;
var
  TheContent: TStringList;
begin
  TheContent:= TStringList.Create;
  try
    fIFrame.Clear;
    fIFrame.Text:= '<iframe src="' + fHeaderFile + '" ' +
      'width="' + IntToStr(fWidth - fScrollBarOffset) + '" ' +
      'height="' + IntToStr(fHeaderHeight) + '" ' +
      'scrolling="no" marginwidth="' + IntToStr(fMarginWidth) + '" ' +
      'marginheight="' + IntToStr(fMarginHeight) + '"></iframe><br>' +
      '<iframe src="' + fBodyFile + '" ' +
      'width="' + IntToStr(fWidth) + '" ' +
      'height="' + IntToStr(fBodyHeight) + '" ' +
      'marginwidth="' + IntToStr(fMarginWidth) + '" ' +
      'marginheight="' + IntToStr(fMarginHeight) + '"></iframe>';

    if fIFrameFile <> '' then
      fIFrame.SaveToFile(IFrameFile);

    fHeader:= true;
    if DataSet <> nil then
    begin
      DataSet.Close;
      DataSet.Open;
    end
    else
    begin
      MessageDlg('No Dataset value !', mtWarning, [mbOK], 0);
      Exit;
    end;

    if fHeaderFile <> '' then
    begin
      TheContent.Text:= fWebScrollTable.Content;
      TheContent.SaveToFile(fHeaderFile);
    end
    else
      MessageDlg('No HeaderFile specified !', mtWarning, [mbOK], 0);

    fHeader:= false;
    DataSet.Close;
    DataSet.Open;

    if fBodyFile <> '' then
    begin
      TheContent.Text:= fWebScrollTable.Content;
      TheContent.SaveToFile(fBodyFile);
    end
    else
      MessageDlg('No BodyFile specified !', mtWarning, [mbOK], 0);
  finally
    TheContent.Free;
  end;
end;

function TmWebScrollTable.GetDataSet: TDataSet;
begin
  Result:= fWebScrollTable.DataSet;
end;

procedure TmWebScrollTable.SetBodyFile(const Value: TFileName);
begin
  fBodyFile:= Value;
end;

procedure TmWebScrollTable.SetCop(Value: string);
begin
  Exit;
end;

procedure TmWebScrollTable.SetDataSet(const Value: TDataSet);
begin
  fWebScrollTable.DataSet:= Value;
end;

procedure TmWebScrollTable.FormatCell(Sender: TObject; CellRow,
  CellColumn: integer; var BgColor: THTMLBgColor; var Align: THTMLAlign;
  var VAlign: THTMLVAlign; var CustomAttrs, CellData: string);
begin
  Align:= haLeft;
  if fHeader then
  begin
    // Headerpart
    if CellRow > 0 then
    begin
      // Write text below headerrow in white on white
      BgColor:= 'White';
      CellData:= '<font color=white>' + CellData;
    end;
  end
  else
  begin
    // Bodypart
    if CellRow = 0 then
    begin
      // Write headerrow in white on white
      BgColor:= 'White';
      CellData:= '<font color=white>' + CellData;
    end;
  end;
  if Assigned(fOnFormatCell) then
    fOnFormatCell(self, fHeader, CellRow, CellColumn, BgColor, Align, VAlign,
      CustomAttrs, CellData);
end;

{ TWebScrollTable }

function TWebScrollTable.Content: string;
var
  AutoActivate: boolean;
begin
  Result:= '';
  if FDataSet <> nil then
  begin
    if FDataSet.Active and (Columns.Count = 0) then LayoutChanged;
    AutoActivate:= not FDataSet.Active;
    if AutoActivate then FDataSet.Open;
    try
      if DoCreateContent then
        Result:= HTMLTable(FDataSet, self, 100000);
    finally
      if AutoActivate then FDataSet.Close;
    end;
  end;
end;

function TWebScrollTable.GetDataSet: TDataSet;
begin
  Result:= FDataSet;
end;

procedure TWebScrollTable.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSet) then
    FDataSet:= nil;
end;

procedure TWebScrollTable.SetDataSet(ADataSet: TDataSet);
begin
  if FDataSet <> ADataSet then
  begin
    if ADataSet <> nil then ADataSet.FreeNotification(self);
    FDataSet:= ADataSet;
    InternalDataSource.DataSet:= FDataSet;
  end;
end;

procedure TmWebScrollTable.SetHeaderFile(const Value: TFileName);
begin
  fHeaderFile:= Value;
end;

procedure TmWebScrollTable.SetIFrameFile(const Value: TFileName);
begin
  fIFrameFile:= Value;
end;

end.

